import os

import ctypes
import tempfile

from pyflagr.RAM import RAM
# from RAM import RAM


# CONDORCET WINNERS METHOD ============================================================================================
class CondorcetWinners(RAM):

    def __init__(self, eval_pts=10):
        RAM.__init__(self, eval_pts)

        self.flagr_lib.Condorcet.argtypes = [
            ctypes.c_char_p,  # Input data file with the lists to be aggregated
            ctypes.c_char_p,  # Input data file with the relevant elements per query - used for evaluation
            ctypes.c_int,     # Number of evaluation points
            ctypes.c_char_p,  # Random string to be embedded into the output file names
            ctypes.c_char_p  # The directory where the output files will be written
        ]

    def aggregate(self, input_file="", input_df=None, rels_file="", rels_df=None, out_dir=None):
        # This is the directory where the output files are written. If nothing is provided, then the preset temp
        # directory of the OS is used. If an invalid path is provided, the aforementioned temp dir is used silently.
        if out_dir is not None and os.path.isdir(out_dir):
            self.output_dir = out_dir
        else:
            self.output_dir = tempfile.gettempdir()

        status = self.check_get_input(input_file, input_df)
        if status != 0:
            return

        status = self.check_get_rels_input(rels_file, rels_df)
        if status != 0:
            return

        ran_str = self.get_random_string(16)

        # Call the exposed Condorcet C function
        self.flagr_lib.Condorcet(
            bytes(self.input_file, 'ASCII'),
            bytes(self.rels_file, 'ASCII'),
            self.eval_pts,
            bytes(ran_str, 'ASCII'),
            bytes(self.output_dir, 'ASCII')
        )

        df_out, df_eval = self.get_output(self.output_dir, ran_str)
        return df_out, df_eval


# COPELAND WINNERS METHOD ============================================================================================
class CopelandWinners(RAM):

    def __init__(self, eval_pts=10):
        RAM.__init__(self, eval_pts)

        self.flagr_lib.Copeland.argtypes = [
            ctypes.c_char_p,  # Input data file with the lists to be aggregated
            ctypes.c_char_p,  # Input data file with the relevant elements per query - used for evaluation
            ctypes.c_int,     # Number of evaluation points
            ctypes.c_char_p,  # Random string to be embedded into the output file names
            ctypes.c_char_p   # The directory where the output files will be written
        ]

    def aggregate(self, input_file="", input_df=None, rels_file="", rels_df=None, out_dir=None):
        # This is the directory where the output files are written. If nothing is provided, then the preset temp
        # directory of the OS is used. If an invalid path is provided, the aforementioned temp dir is used silently.
        if out_dir is not None and os.path.isdir(out_dir):
            self.output_dir = out_dir
        else:
            self.output_dir = tempfile.gettempdir()

        status = self.check_get_input(input_file, input_df)
        if status != 0:
            return

        status = self.check_get_rels_input(rels_file, rels_df)
        if status != 0:
            return

        ran_str = self.get_random_string(16)

        # Call the exposed Copeland C function
        self.flagr_lib.Copeland(
            bytes(self.input_file, 'ASCII'),
            bytes(self.rels_file, 'ASCII'),
            self.eval_pts,
            bytes(ran_str, 'ASCII'),
            bytes(self.output_dir, 'ASCII')
        )

        df_out, df_eval = self.get_output(self.output_dir, ran_str)
        return df_out, df_eval


# OUTRANKING APPROACH =================================================================================================
class OutrankingApproach(RAM):
    preference_t = 0.0
    veto_t = 0.0
    concordance_t = 0.0
    discordance_t = 0.0

    def __init__(self, eval_pts=10, preference=0.0, veto=0.75, concordance=0.0, discordance=0.25):
        RAM.__init__(self, eval_pts)

        self.preference_t = preference
        self.veto_t = veto
        self.concordance_t = concordance
        self.discordance_t = discordance

        self.flagr_lib.OutrankingApproach.argtypes = [
            ctypes.c_char_p,  # Input data file with the lists to be aggregated
            ctypes.c_char_p,  # Input data file with the relevant elements per query - used for evaluation
            ctypes.c_int,     # Number of evaluation points
            ctypes.c_char_p,  # Random string to be embedded into the output file names
            ctypes.c_char_p,  # The directory where the output files will be written
            ctypes.c_float,   # Preference Threshold
            ctypes.c_float,   # Veto Threshold
            ctypes.c_float,   # Concordance Threshold
            ctypes.c_float    # Discordance Threshold
        ]

    def aggregate(self, input_file="", input_df=None, rels_file="", rels_df=None, out_dir=None):
        # This is the directory where the output files are written. If nothing is provided, then the preset temp
        # directory of the OS is used. If an invalid path is provided, the aforementioned temp dir is used silently.
        if out_dir is not None and os.path.isdir(out_dir):
            self.output_dir = out_dir
        else:
            self.output_dir = tempfile.gettempdir()

        status = self.check_get_input(input_file, input_df)
        if status != 0:
            return

        status = self.check_get_rels_input(rels_file, rels_df)
        if status != 0:
            return

        ran_str = self.get_random_string(16)

        # Call the exposed OutrankingApproach C function
        self.flagr_lib.OutrankingApproach(
            bytes(self.input_file, 'ASCII'),
            bytes(self.rels_file, 'ASCII'),
            self.eval_pts,
            bytes(ran_str, 'ASCII'),
            bytes(self.output_dir, 'ASCII'),
            float(self.preference_t),
            float(self.veto_t),
            float(self.concordance_t),
            float(self.discordance_t)
        )

        df_out, df_eval = self.get_output(self.output_dir, ran_str)
        return df_out, df_eval



###############################################################################################################
# Required Python modules and libraries
import Linear
import Majoritarian
import MarkovChains
import Kemeny
import RRA
import Weighted

from sys import platform

import pandas as pd

if __name__ == '__main__':
    base_path = ''
    if platform == "linux" or platform == "linux2":
        base_path = '/media/leo/B65266EC5266B133/phd_Research/08 - Datasets/TREC/Synthetic/'
    elif platform == "win32":
        base_path = 'D:/phd_Research/08 - Datasets/TREC/Synthetic/'
    else:
        exit(1)

    lists = base_path + 'MOSO.csv'
    qrels = base_path + 'MOSO_qrels.csv'

    input_dataframe = pd.read_csv(lists)
    rels_dataframe = pd.read_csv(qrels)

    # method = Linear.BordaCount(eval_pts=20)
    # method = Linear.SimpleBordaCount(eval_pts=20)
    # method = Linear.CombSUM(eval_pts=20, norm="rank")
    # method = Linear.CombSUM(eval_pts=20, norm="score")
    # method = Linear.CombMNZ(eval_pts=20, norm="borda")
    # method = Linear.CombMNZ(eval_pts=20, norm="simple-borda")
    # method = Majoritarian.CondorcetWinners(eval_pts=20)
    # method = Majoritarian.CopelandWinners(eval_pts=20)
    # method = Majoritarian.OutrankingApproach(eval_pts=20)
    # method = MarkovChains.MC4(eval_pts=20)
    # method = RRA.RRA(eval_pts=20)
    # method = RRA.RRA(eval_pts=20, exact=True)
    # method = Weighted.PreferenceRelationsGraph(eval_pts=20)
    # method = Weighted.Agglomerative(eval_pts=20)
    # method = Weighted.DIBRA(eval_pts=20)
    # method = Weighted.DIBRA(eval_pts=20, gamma=1.5, prune=True, d1=0.3, d2=0.05)
    # method = Weighted.DIBRA(eval_pts=20, aggregator="outrank")

    df_out, df_eval = method.aggregate(input_file=lists, rels_file=qrels, out_dir='/home/leo/Documents')

    print(df_eval)



import os

import ctypes
import random
import string
import tempfile
from sys import platform

import pandas as pd


class RAM:
    input_file = ""
    input_df = None
    rels_file = ""
    rels_df = None
    eval_pts = 10
    output_dir = ""
    flagr_lib = ""

    def __init__(self, eval_pts):
        self.eval_pts = eval_pts

        self.flagr_lib = None

        # Import the FLAGR shared library in PyFLAGR
        if platform == "linux" or platform == "linux2":
            self.flagr_lib = ctypes.CDLL(os.path.dirname(os.path.realpath(__file__)) + "/flagr.so")

        elif platform == "win32":
            '''
            os.environ['PATH'] = os.path.dirname(os.path.realpath(__file__)) + os.pathsep + os.environ['PATH']
            paths = os.environ['PATH'].split(";")
            for path in paths:
                if os.path.isdir(path):
                    os.add_dll_directory(path)
            '''
            self.flagr_lib = ctypes.CDLL(os.path.dirname(os.path.realpath(__file__)) + '/flagr.dll')

        elif platform == "darwin":
            self.flagr_lib = ctypes.CDLL(os.path.dirname(os.path.realpath(__file__)) + "/flagr.dylib")

    # Check the input file or DataFrame that contains the input lists
    def check_get_input(self, f, df):
        status = 0
        if len(f) > 0:
            self.input_file = f
            if not os.path.isfile(self.input_file):
                print("Error! Input file does not exist")
                status = -1

        elif df is not None:
            self.input_file = tempfile.gettempdir() + "/temp_input.csv"
            df.to_csv(self.input_file, index=False)

        else:
            print("Error! No input data was passed")
            status = -1

        return status

    # Check the input file or DataFrame that contains the Relevant elements for each query
    def check_get_rels_input(self, rf, rdf):
        status = 0
        if len(rf) > 0:
            self.rels_file = rf

        elif rdf is not None:
            self.rels_file = tempfile.gettempdir() + "/temp_input_rels.csv"
            rdf.to_csv(self.rels_file, index=False)

        else:
            self.rels_file = ""

        return status

    # A random bytes generator - to be used in filenames passed to FLAGR
    def get_random_string(self, length):
        letters = string.ascii_lowercase
        result_str = ''.join(random.choice(letters) for _ in range(length))
        return result_str

    # Retrieve the output from a PyFLAGR rank aggregation method. This one reads the output CSV files and returns
    # two DataFrames. The first one contains the aggregate lists for each query; the second one stores the results
    # of the evaluation (provided that a qrels file or DataFrame has been set).
    def get_output(self, od, ran):
        out_file = od + "/out_" + ran + ".csv"
        eval_file = od + "/eval_" + ran + ".csv"

        if os.path.isfile(out_file):
            df_out = pd.read_csv(out_file, engine='c')
            if od == tempfile.gettempdir():
                os.remove(out_file)

            if os.path.isfile(eval_file):
                df_eval = pd.read_csv(eval_file)
                if od == tempfile.gettempdir():
                    os.remove(eval_file)

                return df_out, df_eval
            else:
                df_rel = pd.DataFrame()
                return df_out, df_rel

        else:
            df_out = pd.DataFrame()
            df_rel = pd.DataFrame()
            return df_out, df_rel



import os

import ctypes
import tempfile

from pyflagr.RAM import RAM
# from RAM import RAM


# PREFERENCE RELATIONS METHOD =========================================================================================
class PreferenceRelationsGraph(RAM):
    Alpha = 0.1
    Beta = 0.5

    def __init__(self, eval_pts=10, alpha=Alpha, beta=Beta):
        RAM.__init__(self, eval_pts)

        self.Alpha = alpha
        self.Beta = beta

        self.flagr_lib.PrefRel.argtypes = [
            ctypes.c_char_p,  # Input data file with the lists to be aggregated
            ctypes.c_char_p,  # Input data file with the relevant elements per query - used for evaluation
            ctypes.c_int,     # Number of evaluation points
            ctypes.c_char_p,  # Random string to be embedded into the output file names
            ctypes.c_char_p,  # The directory where the output files will be written
            ctypes.c_float,   # alpha parameter
            ctypes.c_float    # beta parameter
        ]

    def aggregate(self, input_file="", input_df=None, rels_file="", rels_df=None, out_dir=None):
        # This is the directory where the output files are written. If nothing is provided, then the preset temp
        # directory of the OS is used. If an invalid path is provided, the aforementioned temp dir is used silently.
        if out_dir is not None and os.path.isdir(out_dir):
            self.output_dir = out_dir
        else:
            self.output_dir = tempfile.gettempdir()

        status = self.check_get_input(input_file, input_df)
        if status != 0:
            return

        status = self.check_get_rels_input(rels_file, rels_df)
        if status != 0:
            return

        ran_str = self.get_random_string(16)

        # Call the exposed PrefRel C function
        self.flagr_lib.PrefRel(
            bytes(self.input_file, 'ASCII'),
            bytes(self.rels_file, 'ASCII'),
            self.eval_pts,
            bytes(ran_str, 'ASCII'),
            bytes(self.output_dir, 'ASCII'),
            self.Alpha,
            self.Beta
        )

        df_out, df_eval = self.get_output(self.output_dir, ran_str)
        return df_out, df_eval


# AGGLOMERATIVE METHOD ===============================================================================================
class Agglomerative(RAM):
    C1 = 0.1
    C2 = 0.5

    def __init__(self, eval_pts=10, c1=C1, c2=C2):
        RAM.__init__(self, eval_pts)

        self.C1 = c1
        self.C2 = c2

        self.flagr_lib.Agglomerative.argtypes = [
            ctypes.c_char_p,  # Input data file with the lists to be aggregated
            ctypes.c_char_p,  # Input data file with the relevant elements per query - used for evaluation
            ctypes.c_int,     # Number of evaluation points
            ctypes.c_char_p,  # Random string to be embedded into the output file names
            ctypes.c_char_p,  # The directory where the output files will be written
            ctypes.c_float,   # c1 parameter
            ctypes.c_float    # c2 parameter
        ]

    def aggregate(self, input_file="", input_df=None, rels_file="", rels_df=None, out_dir=None):
        # This is the directory where the output files are written. If nothing is provided, then the preset temp
        # directory of the OS is used. If an invalid path is provided, the aforementioned temp dir is used silently.
        if out_dir is not None and os.path.isdir(out_dir):
            self.output_dir = out_dir
        else:
            self.output_dir = tempfile.gettempdir()

        status = self.check_get_input(input_file, input_df)
        if status != 0:
            return

        status = self.check_get_rels_input(rels_file, rels_df)
        if status != 0:
            return

        ran_str = self.get_random_string(16)

        # Call the exposed Agglomerative C function
        self.flagr_lib.Agglomerative(
            bytes(self.input_file, 'ASCII'),
            bytes(self.rels_file, 'ASCII'),
            self.eval_pts,
            bytes(ran_str, 'ASCII'),
            bytes(self.output_dir, 'ASCII'),
            self.C1,
            self.C2
        )

        df_out, df_eval = self.get_output(self.output_dir, ran_str)
        return df_out, df_eval


# DIBRA METHOD : DISTANCE-BASED ITERATIVE =============================================================================
class DIBRA(RAM):
    agg = 5100
    weight_norm = 2
    distance_metric = 3
    list_pruning = False,
    g = 1.2,
    d_1 = 0.4
    d_2 = 0.1
    tolerance = 0.01
    miter = 50
    preference_t = 0.0
    veto_t = 0.75
    concordance_t = 0.0
    discordance_t = 0.25

    def __init__(self, eval_pts=10, aggregator='combsum:borda', w_norm='minmax', dist='cosine', prune=False, gamma=1.5,
            d1=0.4, d2=0.1, tol=0.01, max_iter=50, pref=0.0, veto=0.75, conc=0.0, disc=0.25):

        RAM.__init__(self, eval_pts)

        # Set the aggregator code according to the selected aggregation method
        self.agg = 5100
        if aggregator == "combsum:borda":
            self.agg = 5100
        elif aggregator == "combsum:rank":
            self.agg = 5101
        elif aggregator == "combsum:score":
            self.agg = 5102
        elif aggregator == "combsum:z-score":
            self.agg = 5103
        elif aggregator == "combsum:simple-borda":
            self.agg = 5104
        elif aggregator == "combmnz:borda":
            self.agg = 5110
        elif aggregator == "combmnz:rank":
            self.agg = 5111
        elif aggregator == "combmnz:score":
            self.agg = 5112
        elif aggregator == "combmnz:z-score":
            self.agg = 5113
        elif aggregator == "combmnz:simple-borda":
            self.agg = 5114
        elif aggregator == "condorcet":
            self.agg = 5200
        elif aggregator == "outrank":
            self.agg = 5300

        # Set the voter weights normalization method
        self.weight_norm = 2
        if w_norm == "none":
            self.weight_norm = 1
        elif w_norm == "minmax":
            self.weight_norm = 2
        elif w_norm == "z":
            self.weight_norm = 3

        # Set the list distance metric
        self.distance_metric = 3
        if dist == "rho":
            self.distance_metric = 1
        elif dist == "cosine":
            self.distance_metric = 3
        elif dist == "footrule":
            self.distance_metric = 4
        elif dist == "tau":
            self.distance_metric = 5

        self.list_pruning = prune
        self.g = gamma
        self.d_1 = d1
        self.d_2 = d2
        self.tolerance = tol
        self.miter = max_iter
        self.preference_t = pref
        self.veto_t = veto
        self.concordance_t = conc
        self.discordance_t = disc

        self.flagr_lib.DIBRA.argtypes = [
            ctypes.c_char_p,  # Input data file with the lists to be aggregated
            ctypes.c_char_p,  # Input data file with the relevant elements per query - used for evaluation
            ctypes.c_int,     # Number of evaluation points
            ctypes.c_int,     # basic un-weighted rank aggregation method
            ctypes.c_char_p,  # Random string to be embedded into the output file names
            ctypes.c_char_p,  # The directory where the output files will be written
            ctypes.c_int,     # Voter weights normalization
            ctypes.c_int,     # List distance function
            ctypes.c_bool,    # List pruning algorithm
            ctypes.c_float,   # gamma parameter
            ctypes.c_float,   # d1 parameter
            ctypes.c_float,   # d2 parameter
            ctypes.c_float,   # convergence precision tolerance
            ctypes.c_int,     # Maximum number of iterations
            ctypes.c_float,   # Preference Threshold
            ctypes.c_float,   # Veto Threshold
            ctypes.c_float,   # Concordance Threshold
            ctypes.c_float    # Discordance Threshold
        ]
        self.flagr_lib.DIBRA.restype = None

    def aggregate(self, input_file="", input_df=None, rels_file="", rels_df=None, out_dir=None):
        # This is the directory where the output files are written. If nothing is provided, then the preset temp
        # directory of the OS is used. If an invalid path is provided, the aforementioned temp dir is used silently.
        if out_dir is not None and os.path.isdir(out_dir):
            self.output_dir = out_dir
        else:
            self.output_dir = tempfile.gettempdir()

        status = self.check_get_input(input_file, input_df)
        if status != 0:
            return

        status = self.check_get_rels_input(rels_file, rels_df)
        if status != 0:
            return

        ran_str = self.get_random_string(16)

        # Call the exposed Agglomerative C function
        self.flagr_lib.DIBRA(
            bytes(self.input_file, 'ASCII'),
            bytes(self.rels_file, 'ASCII'),
            self.eval_pts,
            self.agg,
            bytes(ran_str, 'ASCII'),
            bytes(self.output_dir, 'ASCII'),
            self.weight_norm,
            self.distance_metric,
            self.list_pruning,
            self.g,
            self.d_1,
            self.d_2,
            self.tolerance,
            self.miter,
            self.preference_t,
            self.veto_t,
            self.concordance_t,
            self.discordance_t
        )

        df_out, df_eval = self.get_output(self.output_dir, ran_str)
        return df_out, df_eval



from distutils.core import setup
from setuptools import find_packages

DESCRIPTION = 'PyFLAGR is a Python package for aggregating ranked preference lists from multiple sources.'
LONG_DESCRIPTION =  'FLAGR is a high performing, open-source, modular and cross-platform library for rank aggregation. '\
                    'The core FLAGR library is written in C++ and implements a wide collection of unsupervised rank ' \
                    'aggregation methods. Its modular design allows third-party programmers to implement their own ' \
                    'algorithms and easily integrate them within the library. FLAGR can be built as a standard ' \
                    'console application, or as a shared library (that is, .so or .dll file). In the second case, it ' \
                    'can be linked from other C/C++ programs, or from programs written in other languages (like ' \
                    'Python, R, PHP, etc.).\n\n' \
                    'In this context, PyFLAGR is a Python library that links to FLAGR and allows an analyst to exploit '\
                    'the efficient FLAGR implementations from a standard Python program.\n\n\n' \
                    'FLAGR Website: [http://flagr.mywork.gr/](http://flagr.mywork.gr/)\n\n' \
                    'GitHub repository: [https://github.com/lakritidis/FLAGR](https://github.com/lakritidis/FLAGR)\n\n'

setup(
    name='pyflagr',
    version='1.0.6',
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    author="Leonidas Akritidis",
    author_email="lakritidis@ihu.gr",
    maintainer="Leonidas Akritidis",
    maintainer_email="lakritidis@ihu.gr",
    packages=find_packages(),
    url='https://github.com/lakritidis/FLAGR',
    install_requires=["pandas"],
    license="Apache",
    keywords=[
        "rank aggregation", "rank fusion", "data fusion", "unsupervised learning", "information retrieval",
        "metasearch", "metasearch engines", "borda count", "condorcet", "kendall", "spearman"],
    py_modules=["flagr"],
    package_data={'': ['flagr.so', 'flagr.dll', 'libgcc_s_seh-1.dll', 'libstdc++-6.dll']}
)



import os

import ctypes
import tempfile

from pyflagr.RAM import RAM
# from RAM import RAM


# KEMENY OPTIMAL AGGREGATION METHOD ===================================================================================
class KemenyOptimal(RAM):

    def __init__(self, eval_pts=10):
        RAM.__init__(self, eval_pts)

        self.flagr_lib.Kemeny.argtypes = [
            ctypes.c_char_p,  # Input data file with the lists to be aggregated
            ctypes.c_char_p,  # Input data file with the relevant elements per query - used for evaluation
            ctypes.c_int,     # Number of evaluation points
            ctypes.c_char_p,  # Random string to be embedded into the output file names
            ctypes.c_char_p   # The directory where the output files will be written
        ]

    def aggregate(self, input_file="", input_df=None, rels_file="", rels_df=None, out_dir=None):
        # This is the directory where the output files are written. If nothing is provided, then the preset temp
        # directory of the OS is used. If an invalid path is provided, the aforementioned temp dir is used silently.
        if out_dir is not None and os.path.isdir(out_dir):
            self.output_dir = out_dir
        else:
            self.output_dir = tempfile.gettempdir()

        status = self.check_get_input(input_file, input_df)
        if status != 0:
            return

        status = self.check_get_rels_input(rels_file, rels_df)
        if status != 0:
            return

        ran_str = self.get_random_string(16)

        # Call the exposed Kemeny C function
        self.flagr_lib.Kemeny(
            bytes(self.input_file, 'ASCII'),
            bytes(self.rels_file, 'ASCII'),
            self.eval_pts,
            bytes(ran_str, 'ASCII'),
            bytes(self.output_dir, 'ASCII')
        )

        df_out, df_eval = self.get_output(self.output_dir, ran_str)
        return df_out, df_eval
